function [bX bH bHX bHH logZ] = HCRFInferCluster(F, G, E, nid, W, V, X, enums)

if nargin < 8
    enums = [];
end

[dA nH nX] = size(W);
nNode = size(F, 1);
nEdge = size(E, 1);
dI = size(V, 1);
nCluster = max(nid);
isTrain = ~isempty(X);

eid = EncodeInt(E, int32(1:nNode), nid);
assert(all(eid(:,1) == eid(:,2)), 'inter-cluster edges should not exist');
eid = eid(:,1);

FW = F*reshape(W, [dA, nH*nX]);
GV = G*reshape(V, [dI, nH*nH]);
if isTrain
    FW = reshape(FW,nNode,nH,nX);
    mpf = 0.5*(max(FW(:,:,X),[],2) + min(FW(:,:,X),[],2));
    FW = reshape(FW,nNode,nH*nX);
else
    mpf = 0.5*(max(FW, [], 2) + min(FW,[],2));
end
mpg = 0.5*(max(GV, [], 2)+min(GV,[],2));
FW = reshape(bsxfun(@plus, FW, -mpf), [nNode, nH, nX]);
GV = reshape(bsxfun(@plus, GV, -mpg), [nEdge, nH, nH]);

samples = cell(1, nCluster);
phis = cell(1, nCluster);
ndx = cell(1, nCluster);
edx = cell(1, nCluster);
ee = cell(1, nCluster);

% inbound
msgX = zeros(nCluster, nX);
for ind = 1:nCluster
    ndx{ind} = int32(find(nid == ind));
    edx{ind} = int32(find(eid == ind));
    ee{ind} = EncodeInt32Array(E(edx{ind},:), ndx{ind}) - 1;
    if ~isempty(enums)
        if isTrain
            samples{ind} = enums{length(ndx{ind})}(1:(end - 1), 1:(nH^length(ndx{ind}))) - 1;
            samples{ind} = [samples{ind}; ones(1,size(samples{ind},2))*(X - 1)];
        else
            samples{ind} = enums{length(ndx{ind})} - 1;
        end
    else
        if isTrain
            samples{ind} = int32(GetCombinations(repmat(nH,1,length(ndx{ind})))) - 1;
            samples{ind} = [samples{ind}; ones(1,size(samples{ind},2))*(X - 1)];
        else
            samples{ind} = int32(GetCombinations([repmat(nH,1,length(ndx{ind})), nX])) - 1;
        end
    end
    
    phis{ind} = exp(hcrf_phi(FW(ndx{ind},:,:), GV(edx{ind},:,:), ee{ind}, samples{ind}));
    msgX(ind, :) = accumarray(samples{ind}(end,:)'+1, phis{ind}, [nX 1])';
end

bX = prod(msgX, 1);
msgH = bsxfun(@times, bX, 1./msgX);
if isTrain
    msgH(:,bX==0)=0;
end
Z = sum(bX);
logZ = log(Z) + sum(mpf) + sum(mpg);
bX = bX./Z;

% outbound
bHX = zeros(nNode, nH, nX);
bHH = zeros(nEdge, nH, nH);
for ind = 1:nCluster
    msg = msgH(ind,:)';
    phi = phis{ind}.*msg(samples{ind}(end,:)+1)./Z;
    [dummy bHX(ndx{ind},:,:) bHH(edx{ind},:,:)] = hcrf_getb(nH, nX, ee{ind}, samples{ind}, phi );
end
bH = sum(bHX, 3);
