function [ Xs, bXs bHXs] = HCRFTest( data, model, options )
%[ results ] = HCRFTest( data, model, options ) HCRF inference
%   data: data to inferece. could be cell array or single structure.
%	model: the trained HCRF model. output of HCRFTrain.
%	type: type of inference. 'sp' for sum-product, or 'mp' for max-product.
%	results: the result.

[enums] = GetOptions(options, 'enums', []);
[nX, nH, dA, dI] = GetOptions(model, {'nX', 'nH', 'dimA', 'dimI'});

nSamples = length(data);
theta = model.theta;

[dummy wIndex vIndex] = HCRFGetParamIndex(nX, nH, dA, dI);
W = reshape(theta(wIndex), dA, nH, nX);
if isempty(vIndex)
    V = 0;
else
    V = reshape(theta(vIndex), dI, nH, nH);
end

bXs = zeros(nSamples,nX);
bHXs = 0;
%parfor ind = 1:nSamples
for ind = 1:nSamples
    d = data{ind};
    
    %P(x,h|y,theta)
    if isempty(d.G) || isempty(d.E)
        %bXs(ind,:) = HCRFInferExact(d.F, d.G, d.E, W, V, [], true, enums);
        [bXs(ind,:) dummy bHX] = HCRFInferExact(d.F, d.G, d.E, W, V, [], true, enums);
    elseif isfield(d, 'NID')
        %bXs(ind,:) = HCRFInferCluster(d.F, d.G, d.E, d.NID, W, V, [], enums);
        [bXs(ind,:) dummy bHX] = HCRFInferCluster(d.F, d.G, d.E, d.NID, W, V, [], enums);
    else
        %bXs(ind,:) = HCRFInferExact(d.F, d.G, d.E, W, V, [], false, enums);
        [bXs(ind,:) dummy bHX] = HCRFInferExact(d.F, d.G, d.E, W, V, [], false, enums);
    end
    bHXs = bHXs + squeeze(sum(bHX,1));
end

Xs = argmax(bXs, 2);
