function [model dev] = HCRFTrain( data, options)

[nX, nH, ridge, init, enums, verbose] = GetOptions(options, ...
    'nX', [], 'nH', [], 'ridge', [], 'init', [], 'enums', [], 'verbose', true);

model = struct('nX',nX,'nH',nH,'ridge',ridge,'dimA',size(data{1}.F, 2),'dimI',size(data{1}.G, 2));

nParams = HCRFGetParamIndex(model);
if isempty(init)
    model.theta = randn(nParams, 1)*1e-3;
else
    assert(length(init) == nParams, 'wrong initialization');
    model.theta = init;
end

%% start optimization

if verbose;
    fprintf(['HCRF: %d parameters with %d ' ...
             'samples. nX=%d, nH=%d, DimA=%d, DimI=%d, Ridge=%g\n'], ...
             length(model.theta), length(data), model.nX, ...
             model.nH, model.dimA, model.dimI, model.ridge);
    tic;
end

func = @(x) HCRFValue(x, data, model, enums);
[theta dev] = minFunc(func, model.theta, options);

if verbose;
    fprintf('HCRF: Training complete. Final dev = %g, ET = %g\n', dev, toc);
end

model.theta = theta;
model.dev = dev;
