function [ v, g ] = HCRFValue( theta, data, model, enums )
%[ v, g ] = HCRFValue( theta ) calculate the value, gradient
%the objective is negative log likelihood

[nX nH dA dI ridge] = GetOptions(model, {'nX', 'nH', 'dimA', 'dimI', 'ridge'});

[dummy wIndex vIndex] = HCRFGetParamIndex(nX, nH, dA, dI);
W = reshape(theta(wIndex), dA, nH, nX);
V = reshape(theta(vIndex), dI, nH, nH);

n = length(data);

Z = zeros(n, 1);
bH = cell(n, 1);
bHH = cell(n, 1);
Znx = zeros(n, 1);
bHXnx = cell(n, 1);
bHHnx = cell(n, 1);

%parfor ind = 1:n
for ind = 1:n  %%jl
    d = data{ind};

    if isempty(d.G) || isempty(d.E)
        [bX bH{ind} bHX bHH{ind} Z(ind)] = HCRFInferExact(d.F, d.G, d.E, W, V, d.L, true, enums);
        [bXnx bHnx bHXnx{ind} bHHnx{ind} Znx(ind)] = HCRFInferExact(d.F, d.G, d.E, W, V, [], true, enums);
    elseif isfield(d, 'NID')
        [bX bH{ind} bHX bHH{ind} Z(ind)] = HCRFInferCluster(d.F, d.G, d.E, d.NID, W, V, d.L, enums);
        [bXnx bHnx bHXnx{ind} bHHnx{ind} Znx(ind)] = HCRFInferCluster(d.F, d.G, d.E, d.NID, W, V, [], enums);
    else
        [bX bH{ind} bHX bHH{ind} Z(ind)] = HCRFInferExact(d.F, d.G, d.E, W, V, d.L, false, enums);
        [bXnx bHnx bHXnx{ind} bHHnx{ind} Znx(ind)] = HCRFInferExact(d.F, d.G, d.E, W, V, [], false, enums);
    end
end
assert(all(Z <= Znx + 1e-7), 'wrong inference');

rW = ridge*W; rW(1,:) = 0;
rV = ridge*V; if ~isempty(V); rV(1,:) = 0; end

v = sum(Znx - Z) + 0.5*(sum(rW(:)) + sum(rV(:)));

if nargout > 1
    d = data{1};
    dhx = bHXnx{1};
    dhx(:,:,d.L) = dhx(:,:,d.L) - bH{1};
    gW = d.F'*reshape(dhx, [size(dhx,1), nH*nX]);
    if ~isempty(d.E)
        dhh = bHHnx{1} - bHH{1};
        gV = d.G'*reshape(dhh, [size(dhh,1), nH*nH]);
    else
        gV = 0;
    end

    for ind = 2:n
        d = data{ind};
        dhx = bHXnx{ind};
        dhx(:,:,d.L) = dhx(:,:,d.L) - bH{ind};
        gW = gW + d.F'*reshape(dhx, [size(dhx,1), nH*nX]);

        if ~isempty(d.E)
            dhh = bHHnx{ind} - bHH{ind};
            gV = gV + d.G'*reshape(dhh, [size(dhh,1), nH*nH]);
        end
    end

    g = [gW(:) + rW(:); gV(:) + rV(:)];
end
