#include "mex.h"
#include "./lib/ex-utils.h"

void CheckInput(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    /* Check for proper number of arguments. */
    if (nrhs != 5 || (!mxIsEmpty(prhs[2]) && mxGetClassID(prhs[2]) != mxINT32_CLASS)
            || mxGetClassID(prhs[3]) != mxINT32_CLASS) {
        mexErrMsgTxt("compute marginals for hcrf.\n \
                Usage: [bX bHX bHH] = hcrf_getb(nH, nX, E(int32), samples(int32, per column), w)\n");
    }
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    CheckInput(nlhs, plhs, nrhs, prhs);
    
    register mwSize nNode, nEdge, nX, nH, n;
    nH = (mwSize)mxGetScalar(prhs[0]);
    nX = (mwSize)mxGetScalar(prhs[1]);
    nEdge = mxGetM(prhs[2]);
    nNode = mxGetM(prhs[3]) - 1;
    n = mxGetN(prhs[3]);
    
    register INT32 *E = (INT32*)mxGetPr(prhs[2]);
    INT32 *samples = (INT32*)mxGetPr(prhs[3]);
    register double *w = NULL;
    register double ww = 0;
    mwSize nw = mxGetNumberOfElements(prhs[4]);
    if (nw == 1)
        ww = mxGetScalar(prhs[4]);
    else
        w = mxGetPr(prhs[4]);
    
    mwSize dims[3];
    dims[0] = 1; dims[1] = nX;
    plhs[0] = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
    double* bX = mxGetPr(plhs[0]);
    dims[0] = nNode; dims[1] = nH; dims[2] = nX;
    plhs[1] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL);
    double* bHX = mxGetPr(plhs[1]);
    dims[0] = nEdge; dims[1] = nH; dims[2] = nH;
    plhs[2] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL);
    double* bHH = mxGetPr(plhs[2]);
    
    register INT32* s;
    register mwSize i, node, edge;
    register INT32 x;
    for (i = 0 ; i < n ; ++ i) {
        s = samples + Index2D(0, i, nNode + 1);
        x = s[nNode];
        if (nw > 1)            ww = w[i];
        
        bX[x] += ww;
        for (node = 0 ; node < nNode ; ++ node)
            bHX[Index3D(node, s[node], x, nNode, nH)] += ww;
        for (edge = 0 ; edge < nEdge ; ++ edge)
            bHH[Index3D(edge, s[E[edge]], s[E[edge + nEdge]], nEdge, nH)] += ww;
    }
}
