#include "mex.h"
#include "./lib/ex-utils.h"

using namespace std;

void CheckInput(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    /* Check for proper number of arguments. */
    if (nrhs != 4 || (!mxIsEmpty(prhs[2]) && mxGetClassID(prhs[2]) != mxINT32_CLASS)
            || mxGetClassID(prhs[3]) != mxINT32_CLASS) {
        mexErrMsgTxt("compute energy for hcrf.\n \
                Usage: samples = hcrf_phi(FW, GV, E(int32), samples(int32, per column))\n");
    }
}

//input the strongly influence relationship matrix, output the selected nodes
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    CheckInput(nlhs, plhs, nrhs, prhs);
    
    register mwSize nNode, nEdge, nX, nH;
    const mwSize* sz = mxGetDimensions(prhs[0]);
    nNode = sz[0];  nH = sz[1];  nX = sz[2];
    nEdge = mxIsEmpty(prhs[1]) || mxIsEmpty(prhs[2]) ? 0 : mxGetM(prhs[2]);
    
    register double *FW = mxGetPr(prhs[0]);
    register double *GV = mxGetPr(prhs[1]);
    register INT32 *E = (INT32*)mxGetPr(prhs[2]);
    INT32* samples = (INT32*)mxGetPr(prhs[3]);
    mwSize n = (mwSize)mxGetN(prhs[3]);
    
    //result
    plhs[0] = mxCreateNumericMatrix(n, 1, mxDOUBLE_CLASS, mxREAL);
    double* P = mxGetPr(plhs[0]);
    
    register INT32* s;
    register double psi;
    register mwSize i, node, edge;
    for (i = 0 ; i < n ; ++ i) {
        psi = 0;
        s = samples + Index2D(0, i, nNode + 1);
        
        for (node = 0 ; node < nNode ; ++ node) {
            psi += FW[Index3D(node, s[node], s[nNode], nNode, nH)];
        }
        for (edge = 0 ; edge < nEdge ; ++ edge) {
            psi += GV[Index3D(edge, s[E[edge]], s[E[edge + nEdge]], nEdge, nH)];
        }
        
        P[i] = psi;
    }
}
