function allweights = svmCrossValidate( allfeatures, classlabels, antibodyids, idx_sda)
% function [protlabels,predlabels,proteins,newallweights] = ...
%     svmCrossValidate( allfeatures, classlabels, antibodyids, idx_sda)

addpath lib/classification
addpath lib/classification/libsvm-mat-2.84-1
addpath lib/results

U = unique(classlabels);

[c b] = hist(classlabels,1:1:max(U));
N = min(3,min(c));



% determine CV splits
[proteins Ipr protlabels] = unique(antibodyids);
classlabels2 = zeros(size(classlabels));
for i=1:length(proteins)
    idx = find(proteins(i)==antibodyids);
    m = mode(classlabels(idx));
    classlabels2(idx) = m;
end
[c b] = hist(classlabels2(Ipr),1:1:max(classlabels2))
% N = min(min(c),10);
if ~exist('Nfolds','var')
    Nfolds = 1:N;
end

splits = partition( classlabels2(Ipr), N, 13);




% splits = partition( classlabels, N, 13);
[trainidx, testidx] = partedsets( splits);

predlabels = zeros(length(classlabels),1);
allweights = zeros(length(classlabels),length(U));
committee = [];
for i=1:N
    trainind = find(ismember(protlabels,trainidx{i}));
    testind = find(ismember(protlabels,testidx{i}));

%     trainind = trainidx{i};
%     testind = testidx{i};

    traindata = allfeatures( trainind,:);
    testdata = allfeatures( testind,:);
    
    trainlabels = classlabels(trainind);
    testlabels = classlabels(testind);
    
    % zscore standardization
    [traindata, testdata, mu, st] = standardizeFeatures( traindata, testdata);

    % normalizing
    traindata = normalizeFeatures( traindata);
    testdata = normalizeFeatures( testdata);
    
    ut = clock;
    ut = [num2str(ut(end-2)) num2str(ut(end-1)) num2str(ut(end))];
    ut(ut=='.') = '_';

    if exist('idx_sda','var')
        idx_feat = idx_sda;
    else
        idx_feat = featureSelection( traindata, trainlabels, 'sda', ut);
    end

    idx = unique(idx_feat);
    
    wopt = classweights( trainlabels);
%     options = ['-s 0 -m 1000 -t 2 -c 2048 -g 4 -b 1' wopt];
    options = ['-s 0 -t 2 -c 256 -g 16 -b 1' wopt];

    model = svmtrain( trainlabels, traindata(:,idx), options);
    [predict_label accuracy weights] = svmpredict( testlabels, testdata(:,idx), model, '-b 1');

    committee(i).model = model;
    committee(i).idx_feat = idx;
    committee(i).options = options;
    
    [a ind] = max(weights,[],2);
    predlabels(testind) = model.Label(ind);
    allweights(testind,model.Label) = weights;
end

[cc uu dd ww] = conmatrix( classlabels, predlabels);

% proteins = unique(antibodyids);
% predlabels = zeros(size(proteins));
% protlabels = zeros(size(proteins));
% newallwegihts = zeros(length(proteins),size(allweights,2));
% for i=1:length(proteins)
%     idx = find(proteins(i)==antibodyids);
%     weights = sum(allweights(idx,:),1);
%     [a label] = max(weights,[],2);
    
%     predlabels(i) = label;
    
%     protlabels(i) = mode(classlabels(idx));
%     newallweights(i,:) = weights/length(idx);
% end

% [cc uu dd ww] = conmatrix( protlabels,predlabels);

