function results_mixed( allweights, classlabels, classes, antibodyids, savecsv, savefigure)

addpath lib/results
addpath lib/system

tmp_ind = zeros(length(classes),2);
for i=1:length(classes)
    tmp_idx = find(classes{i}==',');
    
    if tmp_idx
        class1 = classes{i}(1:tmp_idx-1);
        class2 = classes{i}(tmp_idx+1:end);
        
        tmp_idx_1 = find(strcmp(class1,classes));
        tmp_idx_2 = find(strcmp(class2,classes));
        
        tmp_ind(i,:) = [tmp_idx_1 tmp_idx_2];
    end
end

index = [];
for i=1:length(classes)
    index{i} = i;
    
    [r c] = find(tmp_ind==i);
    
    index{i} = [index{i}; tmp_ind(i,:)'; r];
    index{i} = unique(index{i});
    index{i}(index{i}==0) = [];
end

for i=1:length(classes)
    classes{i}(1:4) = [];

    tmp_idx = find(classes{i}==',');
    if tmp_idx
        classes{i} = [classes{i}(1:tmp_idx-1) ' & ' classes{i}(tmp_idx+1:end)];
    end

    
    tmp_idx = findstr('loc_',classes{i});
    classes{i}(tmp_idx:tmp_idx+3) = [];

    tmp_idx = find(classes{i}=='_');
    classes{i}(tmp_idx) = ' ';
end





fid = fopen(savecsv,'w');

fwrite(fid,['Protein level confusion matrix,' char(10)]);
proteins = unique(antibodyids);
predlabels = zeros(size(proteins));
protlabels = zeros(size(proteins));
for i=1:length(proteins)
    idx = find(proteins(i)==antibodyids);
    weights = sum(allweights(idx,:),1);
    [a label] = max(weights,[],2);
    
    predlabels(i) = label;
    
    protlabels(i) = mode(classlabels(idx));
end

classlabels = protlabels;

[C U D W] = conmatrix( classlabels,predlabels);


correctlabels = zeros(size(classlabels));
for i=1:length(classlabels)
    tmp_ind = index{classlabels(i)};
    
    tmp_val = sum(predlabels(i)==tmp_ind);
    if tmp_val
        correctlabels(i) = 1;
    end
end

save('tmpo1.mat','index','predlabels','classes','tmp_ind','classlabels','correctlabels');


Ntot = sum(C(:));
D = round(D*1000)/10;
U = round(U*1000)/10;
W = round(W*1000)/10;

fwrite(fid,',');
for i=1:length(classes)
    fwrite(fid,classes{i});
    fwrite(fid,',');
end
fwrite(fid,['#samples,' char(10)]);

for i=1:length(classes)
    fwrite(fid,classes{i});
    fwrite(fid,',');
    for j=1:length(classes)
        fwrite(fid,num2str(D(i,j)));
        fwrite(fid,',');
    end
    fwrite(fid, [num2str(sum(C(i,:))) ',' char(10)]);
end
fwrite(fid,['Accuracy,' num2str(U) ',' char(10)]);
fwrite(fid,['Class weighted accuracy,' num2str(W) ',' char(10)]);

Uc = round(sum(correctlabels) / length(correctlabels)*1000)/10;
fwrite(fid,['Corrected accuracy,' num2str(Uc) ',' char(10)]);


fclose('all');







[val predlabels] = max(allweights, [], 2);
[s ind] = sort(val);

pr2 = zeros(length(s),1);
re = zeros(length(s),1);
pr = zeros(length(s),1);
numcl = zeros(length(s),1);
for i=length(s):-1:1
    prl = predlabels(ind(i:end));
    cll = classlabels(ind(i:end))';

    pr(i) = sum(prl'==cll)/length(prl);
    re(i) = pr(i)*length(prl)/Ntot;
end



xmin = min(re)*100;
xmax = max(re)*100;
xmax = xmax + 0.1*(xmax-xmin);
ymin = min(pr)*100;
ymax = max(pr)*100;
ymax = ymax + 0.1*(ymax-ymin);

close('all');
figure1 = figure('Color',[1 1 1],'Position', [100 100 650 650]);
axes('Parent',figure1,'FontSize',28,'LineWidth',3, 'XTick',[0 0.2 0.4 0.6 0.8 1]*100, 'YTick',[0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1]*100, 'Position',[.20 .20 .75 .75]);
xpos = [(xmax-xmin)/2+xmin ymin-(ymax-ymin)*.15];
ypos = [-5 (ymax-ymin)/2+ymin];

plot(re*100,pr*100,'k^','LineWidth',2,'Color',[0 0 0]);
xlabel('Recall (%)','FontSize',32,'Position',xpos);% ,'FontName','Kartika');
ylabel('Precision (%)','FontSize',32,'Position',ypos);% ,'FontName','Kartika');
axis([xmin xmax ymin ymax]);

print('-f1', '-depsc', savefigure);



return
