function [edges] = KNN( X, Y, K, verbose )
%[ edges ] = KNN( X, Y, K ) 
%   find the knn of X in Y. brutle force.
%	X/Y: each col is a sample. If only two inputs provided, then Y=X. Note that in this
%	case the self-edges are also added
%	K: K in knn
%
%   edges: result edges, (n*K)*3: (x, y, dist)

if nargin < 4
    verbose = false;
    if nargin < 3
        K = Y;
        Y = X;
        if nargin < 2
            error('not enough input');
        end
    end
end

[nx, dim] = size(X);
ny = size(Y, 2);

maxMem = 100;
batchSize = floor(maxMem*131072/ny);

edges = zeros(nx*K, 3);
tmp = repmat(1:nx, K, 1);
edges(:, 1) = tmp(:);

if verbose;fprintf('Finding %dNN in %d points for %d x %d data... ', K, ny, nx, dim);end

sI = 1;%start index
X = X';
Y = Y';
while sI <= nx
    eI = min(sI + batchSize, nx);

    dist = L2Dist(Y, X(:, sI:eI), false);
    [dist minI]=sort(dist, 1, 'ascend');
    result = [minI(1:K, :), dist(1:K, :)];
    edges(((sI-1)*K + 1):(eI*K), 2:3) = reshape(result, (eI - sI + 1)*K, 2);

    sI = eI + 1;
    
    if verbose;fprintf('%d.',eI);end
end
edges(:,3) = sqrt(max(0, edges(:,3)));

if verbose; fprintf(' complete.\n'); end
