function [D] = L2Dist(A, B, sq)
% [D] = L2Dist(A, B, sq)
%    A - (DxM) matrix 
%    B - (DxN) matrix
%    D - (MxN) Euclidean distances between vectors in A and B
%    D = ||A-B|| = sqrt ( ||A||^2 + ||B||^2 - 2*A.B )
% if not sq: 1) the result is the squared distance. 2) the distance might be
% negative

if ~isreal(A) || ~isreal(B)
   warning('ex:EncDist2', 'cannot handle complex numbers'); 
end

if isempty(B)
    AA = sum(A.^2, 1); 
    D = bsxfun(@plus, bsxfun(@plus, -2*(A'*A), AA'), AA);
else
    D = bsxfun(@plus, bsxfun(@plus, -2*(A'*B), ...
        sum(A.^2, 1)'), sum(B.^2, 1));
end

if nargin < 3 || sq
    D = sqrt(max(0, D));
end
