function [ edges ] = MakeKNNGraph( D, K, mustConn )
%[ edges ] = MakeKNNGraph( D, K, mustConn )
%   make edges in sparse graphs using knn rule. brutle force.
%	D: n x d, each row is a sample
%	K: K in knn
%   mustConn: if force the graph to have only one connected component
%   edges: result edges, nEdge*3: (x, y, distance)

if nargin < 3
    mustConn = false;
end

[n, dim] = size(D);

% find the edges
edges = KNN(D, K + 1);
% remove duplicates
edges = RemoveDupEdges(edges);
% remove self-links
edges = edges(edges(:,1) ~= edges(:,2), :);

if mustConn
    W = sparse(edges(:,1), edges(:,2), edges(:,3), n, n);
    [S, C] = graphconncomp(W, 'Directed', true, 'Weak', true);
    if S > 1%if isolated components exist
        comps = accumarray(C(:), 1:length(C), [S 1], @(X) {x});
        
        nExtraLinks=round(S*(S - 1)/2);
        nOldLinks=size(edges, 1);
        edges(nOldLinks + nExtraLinks, 1)=0;%resize it
        % add one edge between each components
        for ind=1:S
            for jnd=(ind+1):S
                nOldLinks = nOldLinks + 1;
                edges(nOldLinks, :)=[comps{ind}(1), comps{jnd}(1)];
            end
        end
    end
end
