function [obj, grad] = pc_obj(x, T, lam)

global pc_F pc_Y
% pc_F is an M x D matrix. each row for a region.
% pc_Y is M x K label matrix. each row for a region.

[M, D] = size(pc_F);
[M, K] = size(pc_Y);

if length(x) ~= D*T*(K + 1)
   error('check the dimension of param!');
end

W = reshape(x(1:(D*T)), D, T);
V = reshape(x((D*T + 1):end), D, T, K);

A = mlogit(pc_F*W);
B = reshape(mlogit(reshape(pc_F*reshape(V, [D, T*K]), [M*T, K])), [M, T, K]);
C = bsxfun(@times, B, A);
P = squeeze(sum(C, 2));

[iy, jy] = find(pc_Y);
obj = -sum(log(1e-100+P(sub2ind(size(P), iy, jy)))) +...
    0.5*lam*(sum(W(:).^2) + sum(V(:).^2));

if nargout > 1
    Bt = sum(bsxfun(@times, B, reshape(full(pc_Y), [M, 1, K])), 3);
    Pt = 1e-100+full(sum(pc_Y.*P, 2));
    BoP = bsxfun(@times, Bt, 1./Pt);
    
    Q = A.*(BoP - 1);
    g_W = pc_F'*Q;
    
    Q = bsxfun(@plus, reshape(full(pc_Y./(P + 1e-100)), [M, 1, K]), -BoP);
    g_V =  pc_F'*reshape(bsxfun(@times, B.*Q, A), [M, T*K]);
        
    grad = -[g_W(:) - lam*W(:); g_V(:) - lam*V(:)];
    
    if nargout > 2
        error('hession not available');
    end
end
