function [ index ] = randm(w, n, deterministic)
% [ index ] = randm( w, n)
% sample from a multinomial
% w: bin probabilities
% n: number of samples to draw
% deterministic: if using deterministic sampling

if nargin < 3
    deterministic = false;
end

if isscalar(w)%this is for uniform, w is the # of categories
    cw=linspace(1./w, 1, w)';
else
    cw=cumsum(w(:));
    if abs(cw(end) - 1) > 1e-10;        error('wrong distribution');    end
end

if deterministic
    p=linspace(1./(n - 1), 1 - 1./(n - 1), n);
else
    p=rand(1, n);
end

d=bsxfun(@minus, cw, p);
[dummy, index]=min((d<=0)*2 + d);
