%% script for generating figure 7, 8 and 9, and table 1 in the paper:
%% Protein Subcellular Location Pattern Classification in Cellular Images Using Latent Discriminative Models,
%% by Liang Xiong, Jieyue Li, Robert F. Murphy and Jeff Schneider.


close all
clear all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure 7
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% HCRF
cd ./HCRF
result_acctrne_cell_all = [];
result_acctene_cell_all = [];
result_acctene_anti_all = [];
result_acctene_antip_all = [];
result_acctrc_cell_all = [];
result_acctec_cell_all = [];
result_acctec_anti_all = [];
result_acctec_antip_all = [];
result_liktene_cell_all = [];
result_liktec_cell_all = [];

for tt = 1:5
    file = ['./tmp/pc-9-',num2str(tt),'/wrap_results.mat'];
    load(file);
    result_acctrne_cell_all = [result_acctrne_cell_all,result_acctrne_cell];
    result_acctene_cell_all = [result_acctene_cell_all,result_acctene_cell];
    result_acctene_anti_all = [result_acctene_anti_all,result_acctene_anti];
    result_acctene_antip_all = [result_acctene_antip_all,result_acctene_antip];
    result_acctrc_cell_all = [result_acctrc_cell_all,result_acctrc_cell];
    result_acctec_cell_all = [result_acctec_cell_all,result_acctec_cell];
    result_acctec_anti_all = [result_acctec_anti_all,result_acctec_anti];
    result_acctec_antip_all = [result_acctec_antip_all,result_acctec_antip];
    result_liktene_cell_all = [result_liktene_cell_all,result_liktene_cell];
    result_liktec_cell_all = [result_liktec_cell_all,result_liktec_cell];
end

cd ..

%% HLR
cd ./HLR
close all
%clear all

load HLR_wrap_results

h = figure; hold on;
errorbar(mean(Acc_anti_all,2),std(Acc_anti_all,0,2),'-ob','LineWidth',2);
errorbar(mean(result_acctec_antip_all,2),std(result_acctec_antip_all,0,2),'-sr','LineWidth',2);
xlabel('Number of latent variables T','Fontsize',14);
ylabel('Cross-validation accuracies on proteins','Fontsize',14);
set(gca,'FontSize',14);
legend('HLR','HCRF');
hold off
print(h, '-dpsc2', '-r600', 'Schneider.223.fig.7.eps');
print(h, '-dtiff', '-r600', 'Schneider.223.fig.7.tif');

cd ..


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% table 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Confusion Matrix
close all
clear all

M = 15990;
nP = 942;
T = 3;
predlabels = zeros(M,1);
allweights = zeros(M,13);
predlabels_anti_p = zeros(nP,1);
for i=1:5
    file=['./HCRF/tmp/pc-9-4/','accss1-',num2str(T),'-',num2str(i),'.mat'];
    load(file)
    predlabels(teI) = t_hcrfne_c;
    allweights(teI,:) = p_hcrfne_c;
    predlabels_anti_p(find(~isnan(t_hcrfne_a_pv))) = t_hcrfne_a_pv(find(~isnan(t_hcrfne_a_pv)));
end

[c1,u1,d1,w1] = conmatrix(labels_c,predlabels);

[protlabels,protpredlabels,proteins,newallweights] = voteAcrossProtein( allweights, labels_c, anti_id);
[c2,u2,d2,w2] = conmatrix(protlabels,protpredlabels);

[protlabels,protpredlabels,proteins] = jl_voteAcrossProtein(labels_c,anti_id,predlabels);
[c3,u3,d3,w3] = conmatrix(protlabels,protpredlabels);

[protlabels,protpredlabels,proteins] = jl_voteAcrossProtein(labels_a,[1:length(unique(anti_id))],predlabels_anti_p);
[c4,u4,d4,w4] = conmatrix(protlabels,protpredlabels);

table1 = d2;  %% table 1 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure 8
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[re1,pr1] = makePRcurve( protlabels(:), newallweights);
Z1 = trapz(re1,pr1);

load ./HLR/wrap_results25
[re2,pr2] = makePRcurve( protlabels(:), newallweights);
Z2 = trapz(re2,pr2);

load ./HLR/structFeat9-KN3-4HLR_  %% liblinear
[re3,pr3] = makePRcurve( protlabels(:), newallweights);
Z3 = trapz(re3,pr3);

h = figure; hold on;
plot(re1, pr1, '-r', 'linewidth', 2);
plot(re2, pr2, '--b', 'linewidth', 2);
plot(re3, pr3, '-.g', 'linewidth', 2);
legend('HCRF','HLR','SVM');
xlabel('Recall','Fontsize',14);
ylabel('Precision','Fontsize',14);
set(gca,'FontSize',14);
hold off
print(h, '-dpsc2', '-r600', 'Schneider.223.fig.8.eps');
print(h, '-dtiff', '-r600', 'Schneider.223.fig.8.tif');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure 9
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% heatmaps
close all
clear all

load ./HLR/structFeat9-KN3-4HLR_ classes classlabels
[a,b] = hist(classlabels, unique(classlabels));
classes = regexprep(classes, 'loc_', '');
classes = regexprep(classes, 'Cytoskeleton_', '');
classes = regexprep(classes, '_filaments', '');
classes = regexprep(classes, 'Endoplasmic_reticulum', 'ER');
classes = regexprep(classes, '_but_not_', ' w/o ');
classes = regexprep(classes, '_', ' ');

% HLR
load ./HLR/PCO_F_all

PCO_F_all = PCO_F_all./repmat(a,[size(PCO_F_all,1),1]);

PCO_F_all = PCO_F_all./repmat(sum(PCO_F_all,2),[1,size(PCO_F_all,2)]);
%PCO_F_all = PCO_F_all/sum(sum(PCO_F_all));

PCO_F_all = PCO_F_all';
PCO_F_all1 = PCO_F_all;
h = figure;
imagesc(PCO_F_all*255)
%colormap(jet(ceil(max(max(PCO_F_all*255)))))
colorbar('YTickLabel',[])
set(gca,'XTick',[1:size(PCO_F_all,2)],'YTick',[1:size(PCO_F_all,1)],'YTickLabel',classes)
xlabel('Latent variables');
saveas(h, 'exp_map_hlr.eps', 'psc2');
print(h, '-dtiff', '-r300', 'Schneider.223.fig.9a.tif');

%HCRF
load ./HCRF/bHXs
clear bHX
PCO_F_all = bHXs;

PCO_F_all = PCO_F_all./repmat(a,[size(PCO_F_all,1),1]);

PCO_F_all = PCO_F_all./repmat(sum(PCO_F_all,2),[1,size(PCO_F_all,2)]);
%PCO_F_all = PCO_F_all/sum(sum(PCO_F_all));

PCO_F_all = PCO_F_all';
h = figure;
imagesc(PCO_F_all*255)
%colormap(jet(ceil(max(max(PCO_F_all*255)))))
colorbar('YTickLabel',[])
set(gca,'XTick',[1:size(PCO_F_all,2)],'YTick',[1:size(PCO_F_all,1)],'YTickLabel',classes)
xlabel('Latent variables');
saveas(h, 'exp_map_hcrf.eps', 'psc2');
print(h, '-dtiff', '-r300', 'Schneider.223.fig.9b.tif');


scrsz = get(0,'ScreenSize');
h = figure('OuterPosition',[0 0 scrsz(3) scrsz(3)*0.32]);
subplot('Position',[0.155 0.15 0.32 0.8]);
imagesc(PCO_F_all1*255)
colorbar('YTickLabel',[])
set(gca,'XTick',[1:size(PCO_F_all,2)],'YTick',[1:size(PCO_F_all,1)],'YTickLabel',classes)
%xlabel({'Latent variables';'(a)'});
xlabel('Latent variables'); 
text('String','(a)','Position',[0.3,-0.3,0],'Units','normalized','FontName','times new roman','FontSize',14,'HorizontalAlignment','center','VerticalAlignment','bottom','HandleVisibility','off');
axis square
subplot('Position',[0.65 0.15 0.32 0.8]);
imagesc(PCO_F_all*255)
colorbar('YTickLabel',[])
set(gca,'XTick',[1:size(PCO_F_all,2)],'YTick',[1:size(PCO_F_all,1)],'YTickLabel',classes)
xlabel('Latent variables');
text('String','(b)','Position',[0.3,-0.3,0],'Units','normalized','FontName','times new roman','FontSize',14,'HorizontalAlignment','center','VerticalAlignment','bottom','HandleVisibility','off');
axis square
saveas(h, 'Schneider.223.fig.9.fig');
print(h, '-dpsc2', '-r300', 'Schneider.223.fig.9.eps');
print(h, '-dtiff', '-r300', 'Schneider.223.fig.9.tif');
